@@ -68,7 +68,7 @@ lyjson_parse_text(struct ly_ctx *ctx, const char *data, unsigned int *len)
     int32_t value;
 
     for (*len = o = 0; data[*len] && data[*len] != '"'; o++) {
-        if (o > BUFSIZE - 3) {
+        if (o > BUFSIZE - 4) {
             /* add buffer into the result */
             if (result) {
                 size = size + o;
@@ -369,7 +369,7 @@ static unsigned int
 json_get_anydata(struct lyd_node_anydata *any, const char *data)
 {
     struct ly_ctx *ctx = any->schema->module->ctx;
-    unsigned int len = 0, c = 0;
+    unsigned int len = 0, c = 0, skip = 0;
     char *str;
 
     if (data[len] == '"') {
@@ -398,10 +398,23 @@ json_get_anydata(struct lyd_node_anydata *any, const char *data)
     do {
         switch (data[len]) {
         case '{':
-            c++;
+            if (!skip) {
+                c++;
+            }
             break;
         case '}':
-            c--;
+            if (!skip) {
+                c--;
+            }
+            break;
+        case '\\':
+            /* when parsing a string, ignore escaped quotes to not end it prematurely */
+            if (skip && data[len + 1]) {
+                len++;
+            }
+            break;
+        case '\"':
+            skip = !skip;
             break;
         default:
             break;
@@ -449,6 +462,17 @@ repeat:
     leaf->value_type = stype->base;
 
     if (data[len] == '"') {
+        if ((leaf->value_type == LY_TYPE_INT8) || (leaf->value_type == LY_TYPE_INT16) ||
+                (leaf->value_type == LY_TYPE_INT32) || (leaf->value_type == LY_TYPE_UINT8) ||
+                (leaf->value_type == LY_TYPE_UINT16) || (leaf->value_type == LY_TYPE_UINT32)) {
+            LOGVAL(ctx, LYE_XML_INVAL, LY_VLOG_LYD, leaf, "JSON data (expected number, but found string)");
+            return 0;
+        }
+        if (leaf->value_type == LY_TYPE_BOOL) {
+            LOGVAL(ctx, LYE_XML_INVAL, LY_VLOG_LYD, leaf, "JSON data (expected boolean, but found string)");
+            return 0;
+        }
+
         /* string representations */
         ++len;
         str = lyjson_parse_text(ctx, &data[len], &r);
@@ -464,6 +488,12 @@ repeat:
         }
         len += r + 1;
     } else if (data[len] == '-' || isdigit(data[len])) {
+        if ((leaf->value_type == LY_TYPE_INT64) || (leaf->value_type == LY_TYPE_UINT64) ||
+                (leaf->value_type == LY_TYPE_STRING)) {
+            LOGVAL(ctx, LYE_XML_INVAL, LY_VLOG_LYD, leaf, "JSON data (expected string, but found number)");
+            return 0;
+        }
+
         /* numeric type */
         r = lyjson_parse_number(ctx, &data[len]);
         if (!r) {
@@ -482,6 +512,11 @@ repeat:
         }
         len += r;
     } else if (data[len] == 'f' || data[len] == 't') {
+        if (leaf->value_type == LY_TYPE_STRING) {
+            LOGVAL(ctx, LYE_XML_INVAL, LY_VLOG_LYD, leaf, "JSON data (expected string, but found boolean)");
+            return 0;
+        }
+
         /* boolean */
         r = lyjson_parse_boolean(ctx, &data[len]);
         if (!r) {
@@ -490,11 +525,20 @@ repeat:
         }
         leaf->value_str = lydict_insert(ctx, &data[len], r);
         len += r;
-    } else if (!strncmp(&data[len], "[null]", 6)) {
-        /* empty */
+    } else if (data[len] == '[') {
+        /* empty '[' WSP 'null' WSP ']' */
+        for (r = len + 1; isspace(data[r]); ++r);
+        if (strncmp(&data[r], "null", 4)) {
+            goto inval;
+        }
+        for (r += 4; isspace(data[r]); ++r);
+        if (data[r] != ']') {
+            goto inval;
+        }
         leaf->value_str = lydict_insert(ctx, "", 0);
-        len += 6;
+        len = r + 1;
     } else {
+inval:
         /* error */
         LOGVAL(ctx, LYE_XML_INVAL, LY_VLOG_LYD, leaf, "JSON data (unexpected value)");
         return 0;
@@ -502,8 +546,7 @@ repeat:
 
     /* the value is here converted to a JSON format if needed in case of LY_TYPE_IDENT and LY_TYPE_INST or to a
      * canonical form of the value */
-    if (!lyp_parse_value(&((struct lys_node_leaf *)leaf->schema)->type, &leaf->value_str, NULL, leaf, NULL, NULL,
-                         1, 0, options & LYD_OPT_TRUSTED)) {
+    if (!lyp_parse_value(&((struct lys_node_leaf *)leaf->schema)->type, &leaf->value_str, NULL, leaf, NULL, NULL, 1, 0)) {
         return 0;
     }
 
@@ -637,7 +680,7 @@ repeat:
     len += r + 1;
     len += skip_ws(&data[len]);
 
-    ret = lyp_fill_attr(parent_module->ctx, NULL, NULL, prefix, name, value, NULL, options, &attr_new);
+    ret = lyp_fill_attr(parent_module->ctx, NULL, NULL, prefix, name, value, NULL, &attr_new);
     if (ret == -1) {
         free(value);
         goto error;
@@ -707,6 +750,8 @@ store_attrs(struct ly_ctx *ctx, struct attr_cont *attrs, struct lyd_node *first,
 
         if (iter->index) {
             flag_leaflist = 1;
+        } else {
+            flag_leaflist = 0;
         }
 
         LY_TREE_FOR(first, diter) {
@@ -764,6 +809,83 @@ error:
     return -1;
 }
 
+/**
+ * @brief Skip subtree (find its end in the input data) of the current JSON item.
+ * @param[in] ctx libyang context for logging
+ * @param[in] parent parent node for logging
+ * @param[in] data input data (pointing to the beginning, @p len is used to go to the current position).
+ * @param[in, out] len Current position in the @p data, will be updated to the end of the element's subtree in the @p data
+ * @retun 0 on success
+ * @return -1 on error.
+ */
+static int
+json_skip_unknown(struct ly_ctx *ctx, struct lyd_node *parent, const char *data, unsigned int *len)
+{
+    int qstr = 0;
+    int objects = 0;
+    int arrays = 0;
+
+    while (data[*len]) {
+        switch (data[*len]) {
+        case '\"':
+            if (qstr) {
+                if (data[(*len) - 1] != '\\')  {
+                    qstr = 0;
+                }
+            } else if (data[(*len) - 1] != '\\') {
+                qstr = 1;
+            } else {
+                LOGVAL(ctx, LYE_XML_INVAL, LY_VLOG_LYD, parent, "JSON data (missing quotation mark for a string data) ");
+                return -1;
+            }
+            break;
+        case '[':
+            if (!qstr) {
+                arrays++;
+            }
+            break;
+        case '{':
+            if (!qstr) {
+                objects++;
+            }
+            break;
+        case ']':
+            if (!qstr) {
+                arrays--;
+            }
+            break;
+        case '}':
+            if (!qstr) {
+                objects--;
+            }
+            break;
+        case ',':
+            if (!qstr && !objects && !arrays) {
+                /* do not eat the comma character */
+                return 0;
+            }
+        }
+
+        if (objects < 0) {
+            if (arrays) {
+                LOGVAL(ctx, LYE_XML_INVAL, LY_VLOG_LYD, parent, "JSON data (missing end-array)");
+                return -1;
+            }
+            return 0;
+        }
+        if (arrays < 0) {
+            if (objects) {
+                LOGVAL(ctx, LYE_XML_INVAL, LY_VLOG_LYD, parent, "JSON data (missing end-object)");
+                return -1;
+            }
+            return 0;
+        }
+        (*len)++;
+    }
+
+    return 0;
+}
+
 static unsigned int
 json_parse_data(struct ly_ctx *ctx, const char *data, const struct lys_node *schema_parent, struct lyd_node **parent,
                 struct lyd_node *first_sibling, struct lyd_node *prev, struct attr_cont **attrs, int options,
@@ -791,6 +913,10 @@ json_parse_data(struct ly_ctx *ctx, const char *data, const struct lys_node *sch
     len++;
 
     str = lyjson_parse_text(ctx, &data[len], &r);
+    if (!str) {
+        goto error;
+    }
+
     if (!r) {
         goto error;
     } else if (data[len + r] != '"') {
@@ -934,8 +1060,16 @@ json_parse_data(struct ly_ctx *ctx, const char *data, const struct lys_node *sch
 
     module = lys_node_module(schema);
     if (!module || !module->implemented || module->disabled) {
-        LOGVAL(ctx, LYE_INELEM, (*parent ? LY_VLOG_LYD : LY_VLOG_NONE), (*parent), name);
-        goto error;
+        if (options & LYD_OPT_STRICT) {
+            LOGVAL(ctx, LYE_INELEM, (*parent ? LY_VLOG_LYD : LY_VLOG_NONE), (*parent), name);
+            goto error;
+        } else {
+            if (json_skip_unknown(ctx, *parent, data, &len)) {
+                goto error;
+            }
+            free(str);
+            return len;
+        }
     }
 
     if (str[0] == '@') {
@@ -1436,7 +1570,7 @@ lyd_parse_json(struct ly_ctx *ctx, const char *data, int options, const struct l
         result = reply_top;
     }
 
-    if (!result) {
+    if (!result && (options & LYD_OPT_STRICT)) {
         LOGERR(ctx, LY_EVALID, "Model for the data to be linked with not found.");
         goto error;
     }
@@ -1452,7 +1586,12 @@ lyd_parse_json(struct ly_ctx *ctx, const char *data, int options, const struct l
         /* action reply */
         act_notif = reply_parent;
     } else if ((options & (LYD_OPT_RPC | LYD_OPT_NOTIF)) && !act_notif) {
-        LOGVAL(ctx, LYE_MISSELEM, LY_VLOG_LYD, result, (options & LYD_OPT_RPC ? "action" : "notification"), result->schema->name);
+        if (result) {
+            LOGVAL(ctx, LYE_MISSELEM, LY_VLOG_LYD, result, (options & LYD_OPT_RPC ? "action" : "notification"),
+                   result->schema->name);
+        } else {
+            LOGVAL(ctx, LYE_MISSELEM, LY_VLOG_NONE, NULL, (options & LYD_OPT_RPC ? "action" : "notification"), "data");
+        }
         goto error;
     }
 
