@@ -75,7 +75,7 @@ xml_data_search_schemanode(struct lyxml_elem *xml, struct lys_node *start, int o
 
 /* logs directly */
 static int
-xml_get_value(struct lyd_node *node, struct lyxml_elem *xml, int editbits, int trusted)
+xml_get_value(struct lyd_node *node, struct lyxml_elem *xml, int editbits)
 {
     struct lyd_node_leaf_list *leaf = (struct lyd_node_leaf_list *)node;
 
@@ -92,7 +92,7 @@ xml_get_value(struct lyd_node *node, struct lyxml_elem *xml, int editbits, int t
 
     /* the value is here converted to a JSON format if needed in case of LY_TYPE_IDENT and LY_TYPE_INST or to a
      * canonical form of the value */
-    if (!lyp_parse_value(&((struct lys_node_leaf *)leaf->schema)->type, &leaf->value_str, xml, leaf, NULL, NULL, 1, 0, trusted)) {
+    if (!lyp_parse_value(&((struct lys_node_leaf *)leaf->schema)->type, &leaf->value_str, xml, leaf, NULL, NULL, 1, 0)) {
         return EXIT_FAILURE;
     }
 
@@ -324,7 +324,7 @@ xml_parse_data(struct ly_ctx *ctx, struct lyxml_elem *xml, struct lyd_node *pare
                 /* NETCONF filter's attributes, which we implement as non-standard annotations,
                  * they are unqualified (no namespace), but we know that we have internally defined
                  * them in the ietf-netconf module */
-                str = "urn:ietf:params:xml:ns:netconf:base:1.0";
+                str = LY_NSNC;
                 filterflag = 1;
             } else {
                 /* garbage */
@@ -334,11 +334,16 @@ xml_parse_data(struct ly_ctx *ctx, struct lyxml_elem *xml, struct lyd_node *pare
             str = attr->ns->value;
         }
 
-        r = lyp_fill_attr(ctx, *result, str, NULL, attr->name, attr->value, xml, options, &dattr);
+        r = lyp_fill_attr(ctx, *result, str, NULL, attr->name, attr->value, xml, &dattr);
         if (r == -1) {
             goto unlink_node_error;
         } else if (r == 1) {
 attr_error:
+            if (!strcmp(attr->name, "default") && !strcmp(attr->ns->value, "urn:ietf:params:xml:ns:yang:ietf-netconf-with-defaults")) {
+                /* we do not need to parse this attribute, just skip it */
+                continue;
+            }
+
             if (options & LYD_OPT_STRICT) {
                 LOGVAL(ctx, LYE_INATTR, LY_VLOG_LYD, *result, attr->name);
                 goto unlink_node_error;
@@ -355,6 +360,7 @@ attr_error:
             if (!dattr->value.string) {
                 /* problem with resolving value as xpath */
                 dattr->value.string = dattr->value_str;
+                lyd_free_attr(ctx, NULL, dattr, 0);
                 goto unlink_node_error;
             }
             lydict_remove(ctx, dattr->value_str);
@@ -453,7 +459,7 @@ attr_error:
     /* type specific processing */
     if (schema->nodetype & (LYS_LEAF | LYS_LEAFLIST)) {
         /* type detection and assigning the value */
-        if (xml_get_value(*result, xml, editbits, options & LYD_OPT_TRUSTED)) {
+        if (xml_get_value(*result, xml, editbits)) {
             goto unlink_node_error;
         }
     } else if (schema->nodetype & LYS_ANYDATA) {
@@ -464,7 +470,7 @@ attr_error:
             xml->child = NULL;
             LY_TREE_FOR(child, next) {
                 next->parent = NULL;
-                lyxml_correct_elem_ns(ctx, next, 1, 1);
+                lyxml_correct_elem_ns(ctx, next, xml, 1, 1);
             }
 
             ((struct lyd_node_anydata *)*result)->value_type = LYD_ANYDATA_XML;
@@ -482,7 +488,7 @@ attr_error:
         }
         *act_notif = *result;
     } else if (schema->nodetype == LYS_NOTIF) {
-        if (!(options & LYD_OPT_NOTIF) || *act_notif) {
+        if (!(options & (LYD_OPT_NOTIF | LYD_OPT_NOTIF_FILTER)) || *act_notif) {
             LOGVAL(ctx, LYE_INELEM, LY_VLOG_LYD, (*result), schema->name);
             LOGVAL(ctx, LYE_SPEC, LY_VLOG_PREV, NULL, "Unexpected notification node \"%s\".", schema->name);
             goto unlink_node_error;
@@ -559,6 +565,8 @@ error:
 API struct lyd_node *
 lyd_parse_xml(struct ly_ctx *ctx, struct lyxml_elem **root, int options, ...)
 {
+    FUN_IN;
+
     va_list ap;
     int r;
     struct unres_data *unres = NULL;
@@ -666,7 +674,8 @@ lyd_parse_xml(struct ly_ctx *ctx, struct lyxml_elem **root, int options, ...)
     }
 
     if ((options & LYD_OPT_RPC)
-            && !strcmp(xmlstart->name, "action") && !strcmp(xmlstart->ns->value, "urn:ietf:params:xml:ns:yang:1")) {
+            && !strcmp(xmlstart->name, "action")
+            && xmlstart->ns && !strcmp(xmlstart->ns->value, LY_NSYANG)) {
         /* it's an action, not a simple RPC */
         xmlstart = xmlstart->child;
         if (options & LYD_OPT_DESTRUCT) {
